<template>
    <div v-if="bankTransfer?.active">
        <label>
            <input v-model="selectedPaymentMethod" type="radio" :value="bankTransfer.slug" />
            <div style="display: inline-grid">
                <span
                    >{{ bankTransfer.description }}
                    <template v-if="discountRate > 0"> (Dto. {{ discountRate * 100 }}%)</template></span
                >
            </div>
        </label>
    </div>
    <div v-if="siPago?.active">
        <label>
            <input v-model="selectedPaymentMethod" type="radio" :value="siPago.slug" />{{ siPago.description }}
        </label>
    </div>
    <div v-if="mercadoPago?.active">
        <label>
            <input v-model="selectedPaymentMethod" type="radio" :value="mercadoPago.slug" />{{
                mercadoPago.description
            }}
        </label>

        <div v-if="selectedPaymentMethod === 'mercado_pago'">
            <form id="mercado-pago-form" @submit.prevent="submitMercadoPago" />
        </div>

        <div v-if="paymentErrorMessage" class="text-red-600 font-semibold mt-4">
            {{ paymentErrorMessage }}
        </div>
    </div>
</template>

<script setup>
import { ref, computed, onMounted, watch } from "vue";
import { cartStore } from "../../../js/stores/cart.js";
import { useParametersStore } from "../../../js/stores/parameters";
import { sendOrderData } from "../../../js/services/orderService.js";
import { defineExpose } from "vue";

const store = cartStore();
const parameters = useParametersStore();

const props = defineProps({
    from: {
        type: String,
        default: "summary",
    },
    paymentMethods: {
        type: Array,
        required: true,
    },
});

const selectedPaymentMethod = ref(store.selectedPaymentMethod);
const discountRate = computed(() => parameters.discountRate);

const cardBrickController = ref(null);
let sdkLoaded = false;

const showMercadoPagoForm = ref(false);

const paymentErrorMessage = ref(null);

const emit = defineEmits(["payment-failed"]);

const loadMercadoPagoSDK = () => {
    return new Promise((resolve) => {
        if (sdkLoaded) return resolve();
        const script = document.createElement("script");
        script.src = "https://sdk.mercadopago.com/js/v2";
        script.onload = () => {
            sdkLoaded = true;
            resolve();
        };
        document.head.appendChild(script);
    });
};

async function loadMercadoPagoForm() {
    await loadMercadoPagoSDK();

    const mp = new window.MercadoPago(process.env.CLIENT_ID, {
        locale: "es-AR",
    });

    const bricks = mp.bricks();

    if (cardBrickController.value) {
        await cardBrickController.value.unmount();
        cardBrickController.value = null;
    }

    cardBrickController.value = await bricks.create("cardPayment", "mercado-pago-form", {
        initialization: {
            amount: store.total || 100,
            payer: {
                email: store.billingEmail || "",
                identification: {
                    number: store.billingDni || "",
                    type: "DNI",
                },
                cardholder: {
                    name: store.billingEmail,
                },
            },
        },
        customization: {
            paymentMethods: {
                maxInstallments: 6,
            },
            visual: {
                hideFormTitle: true,
                style: {
                    theme: "default",
                    formBackgroundColor: "none",
                    formPadding: 0,
                },
            },
        },
        callbacks: {
            onReady: () => {
                const wrapper = document.querySelector("[data-testid='submit-wrapper']");
                if (wrapper) {
                    wrapper.style.visibility = "hidden";
                    wrapper.style.margin = "0";
                    wrapper.style.height = "0";
                }

                const innerForm = document.querySelector("#mercado-pago-form form");
                if (innerForm) {
                    innerForm.style.padding = "15px 0";
                }
            },
            onSubmit: (cardFormData) => {
                return new Promise((resolve, reject) => {
                    fetch("/payment-mercado-pago-webhook", {
                        method: "POST",
                        headers: { "Content-Type": "application/json" },
                        body: JSON.stringify({
                            token: cardFormData.token,
                            installments: cardFormData.installments,
                            paymentMethodId: cardFormData.payment_method_id,
                            issuerId: cardFormData.issuer_id,
                            payer: { email: store.billingEmail },
                            amount: store.getTotalCart(),
                        }),
                    })
                        .then((res) => res.json())
                        .then(async (result) => {
                            if (result.status === "approved") {
                                store.updateExtraData(result);
                                const order = {
                                    items: store.items,
                                    shipping: store.selectedShippingMethod,
                                    payment: store.selectedPaymentMethod,
                                    billingName: store.billingName,
                                    billingSurname: store.billingSurname,
                                    billingDni: store.billingDni,
                                    billingEmail: store.billingEmail,
                                    billingPhone: store.billingPhone,
                                    billingCity: store.billingCity,
                                    billingAddress: store.billingAddress,
                                    billingNumber: store.billingNumber,
                                    billingDpto: store.billingDpto,
                                    billingFloor: store.billingFloor,
                                    billingPostalCode: store.billingPostalCode,
                                    wantInvoice: store.wantInvoice,
                                    invoicingCuit: store.invoicingCuit,
                                    invoicingCompanyName: store.invoicingCompanyName,
                                    invoicingAddress: store.invoicingAddress,
                                    invoicingCity: store.invoicingCity,
                                    address: store.address,
                                    number: store.number,
                                    dpto: store.dpto,
                                    floor: store.floor,
                                    receiveDni: store.receiveDni,
                                    receiveEmail: store.receiveEmail,
                                    receiveName: store.receiveName,
                                    receiveSurname: store.receiveSurname,
                                    receivePhone: store.receivePhone,
                                    selectedBranch: store.selectedBranch,
                                    selectedCity: store.selectedCity,
                                    shippingPostalCode: store.shippingPostalCode,
                                    selectedProvince: store.selectedProvince,
                                    changeShipping: store.changeShipping,
                                    notes: store.notes,
                                    extraData: store.extraData,
                                };
                                await sendOrderData(order, store);
                                resolve();
                            } else {
                                paymentErrorMessage.value = getStatusDetailMessage(result.status_detail);
                                paymentErrorMessage.value = getStatusDetailMessage(result.status_detail);
                                emit("payment-failed"); // EMITIR DIRECTAMENTE
                                reject(new Error("Pago rechazado")); // RECHAZAR CON UN ERROR
                            }
                        })
                        .catch((error) => {
                            console.error("Error al procesar pago", error);
                            emit("payment-failed"); // EMITIR DIRECTAMENTE
                            reject(error); // RECHAZAR CON EL ERROR ORIGINAL
                        });
                });
            },
            onError: (error) => {
                console.error("Error al procesar pago", error);
                emit("payment-failed"); // EMITIR DIRECTAMENTE
                reject(error); // RECHAZAR CON EL ERROR ORIGINAL
            },
        },
    });
}

function submitMercadoPago() {
    const mpButton = document.querySelector("[data-testid='submit-button'] button");
    if (mpButton) {
        mpButton.click();
    } else {
        console.error("Botón de Mercado Pago no encontrado");
    }
}

function getStatusDetailMessage(statusDetail) {
    const messages = {
        cc_rejected_bad_filled_card_number: "Revisá el número de la tarjeta.",
        cc_rejected_bad_filled_date: "Revisá la fecha de vencimiento.",
        cc_rejected_bad_filled_other: "Revisá los datos ingresados.",
        cc_rejected_bad_filled_security_code: "Revisá el código de seguridad.",
        cc_rejected_blacklist: "No pudimos procesar tu pago. Usá otro medio de pago.",
        cc_rejected_call_for_authorize: "Tenés que autorizar este pago con el banco.",
        cc_rejected_card_disabled: "La tarjeta está inhabilitada. Contactá a tu banco.",
        cc_rejected_card_error: "No pudimos procesar el pago. Intentá de nuevo.",
        cc_rejected_duplicated_payment: "Ya hiciste un pago por ese valor.",
        cc_rejected_high_risk: "El pago fue rechazado por riesgo. Probá con otro medio.",
        cc_rejected_insufficient_amount: "La tarjeta no tiene fondos suficientes.",
        cc_rejected_invalid_installments: "No se puede pagar en esa cantidad de cuotas.",
        cc_rejected_max_attempts: "Llegaste al límite de intentos. Probá más tarde.",
        cc_rejected_other_reason: "No pudimos procesar el pago. Probá con otra tarjeta.",
    };
    return messages[statusDetail] || "No pudimos procesar el pago. Intentá nuevamente.";
}

watch(selectedPaymentMethod, async (newVal) => {
    store.selectedPaymentMethod = newVal;
    store.updatePaymentMethod(newVal);
    paymentErrorMessage.value = null;
    if (props.from === "billing" && newVal === "mercado_pago") {
        showMercadoPagoForm.value = true;
        await loadMercadoPagoForm();
    } else {
        showMercadoPagoForm.value = false;
        if (cardBrickController.value) {
            await cardBrickController.value.unmount();
            cardBrickController.value = null;
        }
    }
});

onMounted(() => {
    if (props.from === "billing" && selectedPaymentMethod.value === "mercado_pago") {
        showMercadoPagoForm.value = true;
        loadMercadoPagoForm();
    }
});

const bankTransfer = computed(() => props.paymentMethods.find((method) => method.slug === "bank_transfer"));

const mercadoPago = computed(() => props.paymentMethods.find((method) => method.slug === "mercado_pago"));

const siPago = computed(() => props.paymentMethods.find((method) => method.slug === "card"));

defineExpose({ submitMercadoPago });
</script>

<style scoped>
label {
    display: block;
    margin: 5px 0;
}

input[type="radio"] {
    margin-right: 10px;
}

form {
    margin-top: 10px;
}

.text-red-600 {
    color: red;
    font-weight: 600;
}

#mercado-pago-form {
    background-color: transparent !important;

    --form-background-color: transparent !important;
}
</style>
